@@ -1,7 +1,6 @@
 /* DWARF 2 Expression Evaluator.
 
    Copyright (C) 2001-2022 Free Software Foundation, Inc.
-   Copyright (C) 2020-2022 Intel Corporation
 
    Contributed by Daniel Berlin (dan@dberlin.org)
 
@@ -115,9 +114,9 @@ struct piece_closure
   /* The pieces describing this variable.  */
   std::vector<dwarf_expr_piece> pieces;
 
-  /* The frame to which a register value is relative, used
+  /* Frame ID of frame to which a register value is relative, used
      only by DWARF_VALUE_REGISTER.  */
-  struct frame_info *frame;
+  struct frame_id frame_id;
 };
 
 /* Allocate a closure for a value formed from separately-described
@@ -136,7 +135,10 @@ allocate_piece_closure (dwarf2_per_cu_data *per_cu,
   c->per_objfile = per_objfile;
   c->per_cu = per_cu;
   c->pieces = std::move (pieces);
-  c->frame = frame;
+  if (frame == nullptr)
+    c->frame_id = null_frame_id;
+  else
+    c->frame_id = get_frame_id (frame);
 
   for (dwarf_expr_piece &piece : c->pieces)
     if (piece.location == DWARF_VALUE_STACK)
@@ -173,9 +175,6 @@ rw_pieced_value (value *v, value *from, bool check_optimized)
     }
   else
     {
-      gdb_assert ((TYPE_LENGTH (value_type (v)) + value_embedded_offset (v))
-		  <= TYPE_LENGTH (value_enclosing_type (v)));
-
       if (check_optimized)
 	v_contents = nullptr;
       else
@@ -219,7 +218,7 @@ rw_pieced_value (value *v, value *from, bool check_optimized)
 	{
 	case DWARF_VALUE_REGISTER:
 	  {
-	    frame_info *frame = c->frame;
+	    frame_info *frame = frame_find_by_id (c->frame_id);
 	    gdbarch *arch = get_frame_arch (frame);
 	    int gdb_regnum = dwarf_reg_to_regnum_or_error (arch, p->v.regno);
 	    ULONGEST reg_bits = 8 * register_size (arch, gdb_regnum);
