@@ -1,6 +1,5 @@
 /* Multi-process/thread control defs for GDB, the GNU debugger.
    Copyright (C) 1987-2022 Free Software Foundation, Inc.
-   Copyright (C) 2017-2022 Intel Corporation
    Contributed by Lynx Real-Time Systems, Inc.  Los Gatos, CA.
    
 
@@ -659,6 +658,9 @@ public:
   /* Return true if thread has active SIMD lanes.  */
   bool is_active ();
 
+  /* Return true if thread is unavailable.  */
+  bool is_unavailable ();
+
   /* Return the current simd lane.  */
   int current_simd_lane ();
 
