@@ -1,7 +1,6 @@
 /* GNU/Linux/x86-64 specific low level interface, for the remote server
    for GDB.
    Copyright (C) 2002-2022 Free Software Foundation, Inc.
-   Copyright (C) 2020 Intel Corporation
 
    This file is part of GDB.
 
@@ -247,7 +246,6 @@ static const int x86_64_regmap[] =
   -1, -1, -1, -1, -1, -1, -1, -1,
   -1, -1, -1, -1, -1, -1, -1, -1,
   -1,					/* pkru  */
-  -1, -1,				/* CET registers CET_U, PL3_SSP.  */
   -1,					/* AMX TILECFG register.  */
   -1,					/* AMX TILEDATA: tmm0 ... tmm7.  */
 };
@@ -281,9 +279,7 @@ static /*const*/ int i386_regmap[] =
 static int
 is_64bit_tdesc (void)
 {
-  struct regcache *regcache = get_thread_regcache (current_thread, 0);
-
-  return register_size (regcache->tdesc, 0) == 8;
+  return register_size (current_process ()->tdesc, 0) == 8;
 }
 
 #endif
@@ -358,7 +354,7 @@ x86_target::low_get_thread_area (int lwpid, CORE_ADDR *addr)
     struct lwp_info *lwp = find_lwp_pid (ptid_t (lwpid));
     gdb_assert (lwp);
     struct thread_info *thr = get_lwp_thread (lwp);
-    struct regcache *regcache = get_thread_regcache (thr, 1);
+    struct regcache *regcache = get_thread_regcache (thr);
     unsigned int desc[4];
     ULONGEST gs = 0;
     const int reg_thread_area = 3; /* bits to scale down register value.  */
@@ -402,71 +398,6 @@ x86_target::low_cannot_fetch_register (int regno)
   return regno >= I386_NUM_REGS;
 }
 
-/* Collect all supported CET registers in GDB's register cache covered by the
-   PTRACE_SETREGSET request with NT_X86_CET flag and store into the
-   process/thread specified by TID.  */
-
-static void
-x86_fill_cet_regs (regcache *regcache, const int tid)
-{
-  /* TODO: Investigate if the x86_check_cet_ptrace_status is required and if so
-     add a proper comment (JIRA/DOQG-3259).  */
-  if (!have_ptrace_getregset || !x86_check_cet_ptrace_status (tid))
-    return;
-
-  uint64_t buf[2] = {0};
-
-  /* If CET indirect branch tracking or shadow stack are enabled, the
-     org.gnu.gdb.i386.cet_u feature has to be supported by the current
-     target.  */
-  collect_register_by_name (regcache, "cet_u", &buf[0]);
-
-  if (tdesc_contains_feature (regcache->tdesc, "org.gnu.gdb.i386.pl3_ssp"))
-    collect_register_by_name (regcache, "pl3_ssp", &buf[1]);
-
-  iovec iov;
-  iov.iov_base = buf;
-  iov.iov_len = sizeof (buf);
-  /* TODO: Test this warning with new kernel and add proper comment
-     (JIRA/DOQG-3253).  */
-  if (ptrace (PTRACE_SETREGSET, tid, NT_X86_CET, &iov) < 0 )
-    perror_warning_with_name (_("Couldn't write CET registers."));
-}
-
-/* Fetch all supported CET registers covered by the PTRACE_GETREGSET request
-   with NT_X86_CET flag from process/thread TID and store their values in GDB's
-   register cache.  */
-
-static void
-x86_store_cet_regs (regcache *regcache, const int tid)
-{
-  /* TODO: Investigate if the x86_check_cet_ptrace_status is required and if so
-  add a proper comment (JIRA/DOQG-3259).  */
-  if (!have_ptrace_getregset || !x86_check_cet_ptrace_status (tid))
-    return;
-
-  uint64_t buf[2] = {0};
-  iovec iov;
-  iov.iov_base = buf;
-  iov.iov_len = sizeof (buf);
-
-  if (ptrace (PTRACE_GETREGSET, tid, NT_X86_CET, &iov) == 0)
-    {
-      /* If CET indirect branch tracking or shadow stack are enabled, the
-	 org.gnu.gdb.i386.cet_u feature has to be supported by the current
-	 target.  */
-      supply_register_by_name (regcache, "cet_u", &buf[0]);
-      if (tdesc_contains_feature (regcache->tdesc, "org.gnu.gdb.i386.pl3_ssp"))
-	supply_register_by_name (regcache, "pl3_ssp", &buf[1]);
-    }
-  else
-    {
-      /* TODO: Test this warning with new kernel and add proper comment
-	 (JIRA/DOQG-3253).  */
-      perror_warning_with_name (_("Failed to get CET registers."));
-    }
-}
-
 static void
 collect_register_i386 (struct regcache *regcache, int regno, void *buf)
 {
@@ -499,9 +430,6 @@ collect_register_i386 (struct regcache *regcache, int regno, void *buf)
 static void
 x86_fill_gregset (struct regcache *regcache, void *buf)
 {
-  if (tdesc_contains_feature (regcache->tdesc, "org.gnu.gdb.i386.cet_u"))
-    x86_fill_cet_regs (regcache, lwpid_of (current_thread));
-
   int i;
 
 #ifdef __x86_64__
@@ -526,9 +454,6 @@ x86_fill_gregset (struct regcache *regcache, void *buf)
 static void
 x86_store_gregset (struct regcache *regcache, const void *buf)
 {
-  if (tdesc_contains_feature (regcache->tdesc, "org.gnu.gdb.i386.cet_u"))
-    x86_store_cet_regs (regcache, lwpid_of (current_thread));
-
   int i;
 
 #ifdef __x86_64__
@@ -1029,14 +954,6 @@ x86_linux_read_description (void)
   if (xcr0_features)
     x86_xcr0 = xcr0;
 
-  bool shstk_enabled = false;
-  bool ibt_enabled = false;
-  if (have_ptrace_getregset == TRIBOOL_TRUE)
-    {
-      shstk_enabled = x86_check_shstk_support ();
-      ibt_enabled = x86_check_ibt_support ();
-    }
-
   if (machine == EM_X86_64)
     {
 #ifdef __x86_64__
@@ -1045,13 +962,11 @@ x86_linux_read_description (void)
       if (xcr0_features)
 	{
 	  tdesc = amd64_linux_read_description (xcr0 & X86_XSTATE_ALL_MASK,
-						!is_elf64, shstk_enabled,
-						ibt_enabled);
+						!is_elf64);
 	}
 
       if (tdesc == NULL)
-	tdesc = amd64_linux_read_description (X86_XSTATE_SSE_MASK, !is_elf64,
-					      shstk_enabled, ibt_enabled);
+	tdesc = amd64_linux_read_description (X86_XSTATE_SSE_MASK, !is_elf64);
       return tdesc;
 #endif
     }
@@ -1060,12 +975,10 @@ x86_linux_read_description (void)
       const target_desc *tdesc = NULL;
 
       if (xcr0_features)
-	  tdesc = i386_linux_read_description (xcr0 & X86_XSTATE_ALL_MASK,
-					       shstk_enabled, ibt_enabled);
+	  tdesc = i386_linux_read_description (xcr0 & X86_XSTATE_ALL_MASK);
 
       if (tdesc == NULL)
-	tdesc = i386_linux_read_description (X86_XSTATE_SSE, shstk_enabled,
-					     ibt_enabled);
+	tdesc = i386_linux_read_description (X86_XSTATE_SSE);
 
       return tdesc;
     }
@@ -3044,8 +2957,7 @@ x86_target::low_supports_range_stepping ()
 int
 x86_target::get_ipa_tdesc_idx ()
 {
-  struct regcache *regcache = get_thread_regcache (current_thread, 0);
-  const struct target_desc *tdesc = regcache->tdesc;
+  const struct target_desc *tdesc = current_process ()->tdesc;
 
 #ifdef __x86_64__
   return amd64_get_ipa_tdesc_idx (tdesc);
