@@ -1,7 +1,6 @@
 /* MI Command Set.
 
    Copyright (C) 2000-2022 Free Software Foundation, Inc.
-   Copyright (C) 2020-2022 Intel Corporation
 
    Contributed by Cygnus Solutions (a Red Hat company).
 
@@ -106,6 +105,9 @@ static bool mi_async = false;
    executing, mi_async is *not* updated.  */
 static bool mi_async_1 = false;
 
+/* User-controllable flag to suppress event notification in MI.  */
+static bool user_wants_mi_suppress_notification = false;
+
 static void
 set_mi_async_command (const char *args, int from_tty,
 		      struct cmd_list_element *c)
@@ -657,11 +659,13 @@ mi_cmd_thread_info (const char *command, char **argv, int argc)
   enum option
     {
       QID_OPT,
+      STOPPED_OPT,
     };
 
   static const struct mi_opt opts[] =
   {
     {"-qid", QID_OPT, 0},
+    {"-stopped", STOPPED_OPT, 0},
     { 0, 0, 0 }
   };
 
@@ -669,6 +673,7 @@ mi_cmd_thread_info (const char *command, char **argv, int argc)
   int oind = 0;
   char *oarg;
   bool enable_qid = false;
+  bool enable_stopped = false;
 
   while (true)
     {
@@ -681,13 +686,16 @@ mi_cmd_thread_info (const char *command, char **argv, int argc)
 	case QID_OPT:
 	  enable_qid = true;
 	  break;
+	case STOPPED_OPT:
+	  enable_stopped = true;
+	  break;
 	}
     }
 
   if (argc - oind > 1)
     error ("-thread-info: too many arguments");
 
-  info_threads_opts it_opts {false, false, enable_qid};
+  info_threads_opts it_opts {false, enable_stopped, enable_qid};
   print_thread_info (current_uiout, argv[oind], -1, it_opts);
 }
 
@@ -2261,7 +2269,7 @@ mi_cmd_execute (struct mi_parse *parse)
 
   gdb_assert (parse->cmd != nullptr);
 
-  gdb::optional<scoped_restore_tmpl<int>> restore_suppress_notification
+  gdb::optional<scoped_restore_tmpl<bool>> restore_suppress_notification
     = parse->cmd->do_suppress_notification ();
 
   parse->cmd->invoke (parse);
@@ -2905,6 +2913,27 @@ mi_cmd_complete (const char *command, char **argv, int argc)
 		       result.number_matches == max_completions ? "1" : "0");
 }
 
+/* Implement 'show suppress-mi-notifications'.  */
+
+static void
+show_suppress_mi_notifications (ui_file *file, int from_tty,
+				cmd_list_element *c, const char *value)
+{
+  fprintf_filtered (file, _("Suppression of printing MI notifications "
+			    "is %s.\n"), value);
+}
+
+/* Implement 'set suppress-mi-notifications'.  */
+
+static void
+set_suppress_mi_notifications (const char *args, int from_tty,
+			       cmd_list_element *c)
+{
+  mi_suppress_notification.user_selected_context
+    = user_wants_mi_suppress_notification;
+  mi_suppress_notification.thread_state
+    = user_wants_mi_suppress_notification;
+}
 
 void _initialize_mi_main ();
 void
@@ -2929,4 +2958,14 @@ Tells GDB whether MI should be in asynchronous mode."),
     = add_alias_cmd ("target-async", mi_async_cmds.show, class_run, 0,
 		     &showlist);
   deprecate_cmd (show_target_async_cmd, "show mi-async");
+
+  add_setshow_boolean_cmd ("suppress-mi-notifications", no_class,
+			   &user_wants_mi_suppress_notification, _("\
+Set whether MI notifications are suppressed."), _("\
+Show whether MI notifications are suppressed."), _("\
+When on, notifications (such as *stopped event)\n\
+in MI are suppressed."),
+			   set_suppress_mi_notifications,
+			   show_suppress_mi_notifications,
+			   &setlist, &showlist);
 }
