@@ -1,7 +1,6 @@
 /* Multi-process/thread control for GDB, the GNU debugger.
 
    Copyright (C) 1986-2022 Free Software Foundation, Inc.
-   Copyright (C) 2017-2022 Intel Corporation
 
    Contributed by Lynx Real-Time Systems, Inc.  Los Gatos, CA.
 
@@ -184,6 +183,17 @@ thread_info::is_active ()
   return active_simd_lanes_mask () != 0;
 }
 
+/* See gdbthread.h.  */
+
+bool
+thread_info::is_unavailable ()
+{
+  /* We cannot access registers of an unavailable thread.
+     Try to read PC to check whether the thread is available.  */
+  return regcache_read_pc_protected (get_thread_regcache (this)) == 0;
+}
+
+
 /* See gdbthread.h.  */
 
 int
@@ -1256,12 +1266,8 @@ should_print_thread (const char *requested_threads, int default_inf_num,
   if (!opts.show_stopped_threads)
     return true;
 
-  /* Otherwise, show only stopped threads whose registers are available.
-     Attempt to read the PC to check for register availability.  */
-  bool is_stopped = (thr->state == THREAD_STOPPED);
-  bool has_pc = regcache_read_pc_protected (get_thread_regcache (thr)) != 0;
-
-  return (is_stopped && has_pc);
+  /* Otherwise, show only stopped threads whose registers are available.  */
+  return (thr->state == THREAD_STOPPED) && !thr->is_unavailable ();
 }
 
 /* Return the string to display in "info threads"'s "Target Id"
@@ -1359,11 +1365,14 @@ print_thread_row (struct ui_out *uiout, thread_info *tp,
     {
       /* The switch above put us at the top of the stack (leaf
 	 frame).  */
-      bool has_pc
-	= regcache_read_pc_protected (get_thread_regcache (tp)) != 0;
-      /* An inactive thread may still has its PC available, which would
-	 give us a stack frame.  */
-      if (is_active || has_pc)
+      bool is_unavailable = tp->is_unavailable ();
+      /* We do not show frame only for unavailable threads, since for such
+	 threads we do not have PC.
+	 Note: inactive threads might still have a valid PC and frame, just
+	 all lanes are inactive, so we cannot read SIMD-dependent values.  */
+      if (is_unavailable)
+	uiout->text ("(unavailable)\n");
+      else
 	{
 	  scoped_restore_current_simd_lane restore_lane {tp};
 	  if (!lanes.empty ())
@@ -1377,13 +1386,19 @@ print_thread_row (struct ui_out *uiout, thread_info *tp,
 			     /* For MI output, print frame level.  */
 			     uiout->is_mi_like_p (),
 			     LOCATION, 0);
-	}
-      else
-	{
-	  /* Lanes in this row are inactive.  This can happen if the current
-	     thread enters a conditional branch or all lanes in thread are
-	     inactive.  */
-	  uiout->text ("(inactive)\n");
+
+	  /* SIMD specific fields for MI.  */
+	  if (uiout->is_mi_like_p () && tp->has_simd_lanes ())
+	    {
+	      unsigned int mask = tp->active_simd_lanes_mask ();
+	      uiout->field_fmt ("execution-mask", "0x%x", mask);
+	      unsigned int width = tp->get_simd_width ();
+	      uiout->field_fmt ("simd-width", "%u", width);
+	      bpstat *bp = tp->control.stop_bpstat;
+	      if (bp != nullptr)
+		uiout->field_fmt ("hit-lanes-mask", "0x%x",
+				  bp->simd_lane_mask);
+	    }
 	}
     }
 
@@ -1402,20 +1417,6 @@ print_thread_row (struct ui_out *uiout, thread_info *tp,
   int core = target_core_of_thread (tp->ptid);
   if (uiout->is_mi_like_p () && core != -1)
     uiout->field_signed ("core", core);
-
-  if (tp->has_simd_lanes () && uiout->is_mi_like_p ()
-      && tp->state != THREAD_RUNNING)
-    {
-      unsigned int mask = tp->active_simd_lanes_mask ();
-      uiout->field_string ("execution-mask", (string_printf (_("0x%x"),
-			   mask)).c_str ());
-      unsigned int width = tp->get_simd_width ();
-      uiout->field_string ("simd-width", (string_printf (_("%d"),
-			   width)).c_str ());
-      bpstat *bp = tp->control.stop_bpstat;
-      if (bp != nullptr)
-	uiout->field_fmt ("hit-lanes-mask", _("0x%x"), bp->simd_lane_mask);
-    }
 }
 
 /* Like print_thread_info, but in addition, GLOBAL_IDS indicates
@@ -1794,6 +1795,11 @@ scoped_restore_current_simd_lane::~scoped_restore_current_simd_lane ()
   if (m_tp == nullptr)
     return;
 
+  /* The current target may have changed.  SIMD lane queries may require
+     target access via the current_inferior's top target.  Do the switch.  */
+  scoped_restore_current_thread restore_thread;
+  switch_to_thread (m_tp.get ());
+
   if (m_simd_lane_num != -1 && m_tp->has_simd_lanes ())
     {
       /* Restore the previous lane if it is active now or
@@ -2265,8 +2271,23 @@ thread_apply_command (const char *tidlist, int from_tty)
       struct inferior *inf;
       int inf_num, thr_num, simd_lane_num;
 
-      bool is_simd_from_star = parser.in_simd_lane_star_range ();
+      /* Find, whether the element, parsed next is from a wildcard (:*)
+	 SIMD range.  */
+      bool is_simd_from_star = false;
+
+      /* If we are in the middle of the SIMD range, read the star-state
+	 before getting the element.  If the element is the last one in
+	 the range, the parser's state will change after the read.  */
+      if (parser.in_simd_lane_state ())
+	is_simd_from_star = parser.in_simd_lane_star_range ();
+
       parser.get_tid (&inf_num, &thr_num, &simd_lane_num);
+
+      /* When the range was just started, we did not set IS_SIMD_FROM_STAR
+	 yet.  Do it now.  */
+      if (!is_simd_from_star && parser.in_simd_lane_state ())
+	is_simd_from_star = parser.in_simd_lane_star_range ();
+
       inf = find_inferior_id (inf_num);
       if (inf != NULL)
 	tp = find_thread_id (inf, thr_num);
@@ -2341,6 +2362,12 @@ thread_apply_command (const char *tidlist, int from_tty)
 	      continue;
 	    }
 
+	  /* If this is the last meaningful lane of this thread, skip
+	     the rest of the SIMD range.  */
+	  if ((simd_lane_num >= (tp->get_simd_width () - 1))
+	      && parser.in_simd_lane_state ())
+	    parser.skip_simd_lane_range ();
+
 	  /* If thread has SIMD lanes, check that the specified one is
 	       currently active.  */
 	  if (tp->is_simd_lane_active (simd_lane_num))
@@ -2412,18 +2439,21 @@ thread_command (const char *tidstr, int from_tty)
 	  std::string status_note = "";
 	  std::vector<int> lanes;
 
-	  if (tp->state == THREAD_STOPPED && tp->has_simd_lanes ())
+	  if (tp->state == THREAD_STOPPED)
 	    {
-	      if (tp->is_active ())
+	      if (tp->is_unavailable ())
+		status_note = " (unavailable)";
+	      else if (tp->has_simd_lanes ())
 		{
-		  int lane = tp->current_simd_lane ();
-		  lane_info = " lane " + std::to_string (lane);
-		  lanes.push_back (lane);
+		  if (tp->is_active ())
+		    {
+		      int lane = tp->current_simd_lane ();
+		      lane_info = " lane " + std::to_string (lane);
+		      lanes.push_back (lane);
+		    }
+		  else
+		    status_note = " (inactive)";
 		}
-	      else if (tp->state == THREAD_EXITED)
-		status_note = " (inactive, exited)";
-	      else
-		status_note = " (inactive)";
 	    }
 	  else if (tp->state == THREAD_EXITED)
 	    status_note = " (exited)";
@@ -2610,17 +2640,24 @@ print_selected_thread_frame (struct ui_out *uiout,
 	  uiout->field_string ("new-thread-id", print_thread_id (tp, &lanes));
 	  uiout->text (" (");
 	  uiout->text (target_pid_to_str (inferior_ptid));
-	  if (tp->state == THREAD_STOPPED && tp->has_simd_lanes ())
+	  if (tp->state == THREAD_STOPPED)
 	    {
-	      if (is_active)
+	      if (tp->is_unavailable ())
+		uiout->text (") unavailable]");
+	      else if (tp->has_simd_lanes ())
 		{
-		  uiout->text (" lane ");
-		  int lane = tp->current_simd_lane ();
-		  uiout->text (std::to_string (lane));
-		  uiout->text (")]");
+		  if (is_active)
+		    {
+		      uiout->text (" lane ");
+		      int lane = tp->current_simd_lane ();
+		      uiout->text (std::to_string (lane));
+		      uiout->text (")]");
+		    }
+		  else
+		    uiout->text (") inactive]");
 		}
 	      else
-		uiout->text (") inactive]");
+		uiout->text (")]");
 	    }
 	  else
 	    uiout->text (")]");
